@@ -1,7 +1,7 @@
-#include "BBoxDeco.hpp"
+#include "BBoxDeco.h"
 
 #include "gl2ps.h"
-#include "glgui.hpp"
+#include "glgui.h"
 #include "scene.h"
 #include <cstdio>
 #include <cmath>
@@ -391,7 +391,7 @@ AABox BBoxDeco::getBoundingBox(const AABox& in_bbox) const
 
 void BBoxDeco::render(RenderContext* renderContext)
 {
-  AABox bbox = renderContext->scene->getBoundingBox();
+  AABox bbox = renderContext->subscene->getBoundingBox();
 
   if (bbox.isValid()) {
   
@@ -422,7 +422,7 @@ void BBoxDeco::render(RenderContext* renderContext)
 
     // transform vertices: used for edge distance criterion and text justification
 
-    Matrix4x4 modelview(renderContext->modelview);
+    Matrix4x4 modelview(renderContext->subscene->modelMatrix);
 
     for(i=0;i<8;i++)
       eyev[i] = modelview * boxv[i];
